using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.VirtualServer.Interop; // Virtual Server 2005 R2 COM API
using System.Runtime.InteropServices; // Supports remote Virtual Server instances (DCOM)

/*
 * *****************************************
 ***  Virtual Server 2005 Test Application
 * *****************************************
 * Anil Desai, 2006
 * (For more information, see http://AnilDesai.net or e-mail Anil@AnilDesai.net)
 * 
 * Notes:  This application is provided as a simple example of various
 *          Virtual Server automation methods.  This code is provided for
 *          demonstratin purposes only.  It does not include any error-handling
 *          and has not been tested for production usage.
 * 
 * */

namespace VirtualServerTest2
{

    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        // Virtual Server object (supports local or remote connection)
        private VMVirtualServerClass objVirtualServer;

        // Virtual Machine object (currently-selected VM)
        private VMVirtualMachine objVirtualMachine;

        [MTAThread]
        private void Form1_Load(object sender, EventArgs e)
        {
            WriteOutput(1, "--- Virtual Server 2005 Test App Started --");
        }


        private void btnListNetworkAdapters_Click(object sender, EventArgs e)
        {
            foreach (VMVirtualMachine objVirtualMachine
                in objVirtualServer.VirtualMachines)
                // Loop through virtual machines
                foreach (VMNetworkAdapter objNetworkAdapter
                    in objVirtualMachine.NetworkAdapters)
                {
                    WriteOutput(1, "Virtual Machine: " +
                        objNetworkAdapter.VirtualMachine.Name.ToString());
                    WriteOutput(2, "Ethernet Address: " +
                        objNetworkAdapter.EthernetAddress);
                }
        }

        private void btnClearOutput_Click(object sender, EventArgs e)
        {
            txtOutput.Text = "";
        }
                
        private void btnCreateVM_Click(object sender, EventArgs e)
        {
            // Create a new virtual machine (prompt user for the name)
            //string VMName = "";
            objVirtualServer.CreateVirtualMachine
                ("New Virtual Machine", "C:\\Temp\\New Virtual Machine");
            PopulateVMList();
        }

        private void btnConnectLocal_Click(object sender, EventArgs e)
        {
            objVirtualServer = new VMVirtualServerClass();
            WriteOutput(1, "Connected to local instance of Virtual Server.");
            WriteOutput(2, "Virtual Server Name: " 
                + objVirtualServer.Name.ToString());
            WriteOutput(2, "Virtual Server Version: " 
                + objVirtualServer.Version.ToString());

            PopulateVMList();
        }

        private void btnConnectRemote_Click(object sender, EventArgs e)
        {
            // Access a remote installation of Virtual Server using DCOM
            // Reference: Virtual Server Programmer's Guide
            Type typeVSClass;
            Type typeDCOM;
            object objDCOM;

            typeVSClass = typeof(VMVirtualServerClass);
            typeDCOM = Type.GetTypeFromCLSID(typeVSClass.GUID, "Kerby", true);
            objDCOM = Activator.CreateInstance(typeDCOM);

            // Create an instance
            objVirtualServer = (VMVirtualServerClass)Marshal.CreateWrapperOfType(objDCOM, typeVSClass);

            // Output the name and version info from remote Virtual Server instance
            WriteOutput(1, "Connected to remote instance of Virtual Server.");
            WriteOutput(2, "Virtual Server Name: " 
                + objVirtualServer.Name.ToString());
            WriteOutput(2, "Virtual Server Version: " 
                + objVirtualServer.Version.ToString());

            PopulateVMList();
        }

        private void PopulateVMList()
        {
            // Update the list of VMs based
            lstVirtualMachines.Items.Clear();

            WriteOutput(1, "Refreshing list of virtual machines...");
            foreach (VMVirtualMachine objVM in objVirtualServer.VirtualMachines)
            {
                lstVirtualMachines.Items.Add(objVM.Name.ToString());
            }
        }

        private void RefreshTabInfo()
        {
            if (lstVirtualMachines.Items.Count == 0) return;

            //Clear all current information
            txtVMName.Text = "";
            txtVMGeneralInfo.Text = "";
            txtVMStatistics.Text = "";
            lstVMHardDisks.Items.Clear();
            txtVHDInfo.Text = "";
            lstNetworkAdapters.Items.Clear();
            txtNetworkDetails.Text = "";

            //Refresh General Tab
            txtVMName.Text = objVirtualMachine.Name.ToString();
            txtVMGeneralInfo.Text = "";
            txtVMGeneralInfo.Text += "\r\n" + "Virtual Machine Name:      " 
                + objVirtualMachine.Name.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "Current State:             " 
                + objVirtualMachine.State.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "Account Name:              " 
                + objVirtualMachine.AccountName.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "CPU Speed (MHz):           " 
                + objVirtualMachine.ProcessorSpeed.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "Memory (MB):               " 
                + objVirtualMachine.Memory.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "# of Disks:                " 
                + objVirtualMachine.HardDiskConnections.Count.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "# of Network Adapters:     " 
                + objVirtualMachine.NetworkAdapters.Count.ToString();
            txtVMGeneralInfo.Text += "\r\n" + "Notes:                     " 
                + objVirtualMachine.Notes.ToString();

            //Refresh Statistics Tab
            UpdateVmStatistics();

            //Refresh Disks Tab
            foreach (IVMHardDiskConnection objVHDConnection 
                in objVirtualMachine.HardDiskConnections)
            {
                lstVMHardDisks.Items.Add(objVHDConnection.HardDisk.File.ToString());
            }

            //Refresh Networks Tab
            foreach (IVMNetworkAdapter objNetworkAdapter 
                in objVirtualMachine.NetworkAdapters)
            {
                lstNetworkAdapters.Items.Add(objNetworkAdapter.EthernetAddress.ToString());
            }

        }

        private void UpdateVmStatistics()
        {
            txtVMStatistics.Text = "";
            txtVMStatistics.Text += "\r\n" + "Uptime (seconds):         " 
                + objVirtualMachine.Accountant.UpTime.ToString();
            txtVMStatistics.Text += "\r\n" + "CPU Utilization:          " 
                + objVirtualMachine.Accountant.CPUUtilization.ToString();
            txtVMStatistics.Text += "\r\n" + "Relative Weight:          " 
                + objVirtualMachine.Accountant.relativeWeight.ToString();
            txtVMStatistics.Text += "\r\n" + "Disk Bytes Read:          " 
                + objVirtualMachine.Accountant.DiskBytesRead.ToString();
            txtVMStatistics.Text += "\r\n" + "Disk Bytes Written:       " 
                + objVirtualMachine.Accountant.DiskBytesWritten.ToString();
            txtVMStatistics.Text += "\r\n" + "Network Bytes Sent:       " 
                + objVirtualMachine.Accountant.NetworkBytesSent.ToString();
            txtVMStatistics.Text += "\r\n" + "Network Bytes Rec'd:      " 
                + objVirtualMachine.Accountant.NetworkBytesReceived.ToString();
        }

        private void lstVirtualMachines_SelectedIndexChanged(object sender, EventArgs e)
        {
            objVirtualMachine = objVirtualServer.FindVirtualMachine(lstVirtualMachines.SelectedItem.ToString());
            WriteOutput(1, "Selected Virtual Machine: " + objVirtualMachine.Name.ToString());
            RefreshTabInfo();
        }

        private void lstVMHardDisks_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Update Disk-related information
            foreach (IVMHardDiskConnection objHardDiskConnection in objVirtualMachine.HardDiskConnections)
            {
                if (objHardDiskConnection.HardDisk.File.ToString() == lstVMHardDisks.SelectedItem.ToString())
                {
                    txtVHDInfo.Text = "";
                    txtVHDInfo.Text += "\r\n" + "Disk Size in Guest (bytes):     "
                        + objHardDiskConnection.HardDisk.SizeInGuest.ToString();
                    txtVHDInfo.Text += "\r\n" + "Bus Type:                       "
                        + objHardDiskConnection.busType.ToString();
                    txtVHDInfo.Text += "\r\n" + "Bus Number:                     "
                        + objHardDiskConnection.busNumber.ToString();
                    txtVHDInfo.Text += "\r\n" + "Device Number:                  "
                        + objHardDiskConnection.deviceNumber.ToString();
                    txtVHDInfo.Text += "\r\n" + "Host Free Disk Space (bytes):   "
                        + objHardDiskConnection.HardDisk.HostFreeDiskSpace.ToString();
                }
            }
        }

        private void btnVMStart_Click(object sender, EventArgs e)
        {
            //Note: No state-checking or error-handling has been implemented
            WriteOutput(1, "Starting Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            objVirtualMachine.Startup();
        }

        private void lstNetworkAdapters_SelectedIndexChanged(object sender, EventArgs e)
        {
            foreach (IVMNetworkAdapter objNetworkAdapter in objVirtualMachine.NetworkAdapters)
            {
                txtNetworkDetails.Text = "";
                txtNetworkDetails.Text += "\r\n" + "Ethernet Address:            "
                    + objNetworkAdapter.EthernetAddress.ToString(); ;
                txtNetworkDetails.Text += "\r\n" + "Virtual Machine Name:        "
                    + objNetworkAdapter.VirtualMachine.Name.ToString();
                txtNetworkDetails.Text += "\r\n" + "Virtual Network:             "
                    + objNetworkAdapter.virtualNetwork.Name.ToString();
                txtNetworkDetails.Text += "\r\n" + "Virtual Network Filename:    "
                    + objNetworkAdapter.virtualNetwork.File.ToString();
                txtNetworkDetails.Text += "\r\n" + "Packets Sent (bytes):        "
                    + objNetworkAdapter.virtualNetwork.PacketsSent.ToString();
                txtNetworkDetails.Text += "\r\n" + "Packets Rec'd (bytes):       "
                    + objNetworkAdapter.virtualNetwork.PacketsReceived.ToString();
            }

        }

        private void btnStatisticsRefresh_Click(object sender, EventArgs e)
        {
            WriteOutput(1, "Refreshing statistics for Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            UpdateVmStatistics();
        }

        private void btnTurnOffVM_Click(object sender, EventArgs e)
        {
            WriteOutput(1, "Turning off Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            objVirtualMachine.TurnOff();
        }

        private void btnVMPause_Click(object sender, EventArgs e)
        {
            WriteOutput(1, "Pausing Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            objVirtualMachine.Pause();
        }

        private void btnVMResume_Click(object sender, EventArgs e)
        {
            WriteOutput(1, "Resuming Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            objVirtualMachine.Resume();
        }

        private void btnVMReset_Click(object sender, EventArgs e)
        {
            WriteOutput(1, "Resetting Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            objVirtualMachine.Reset();
        }

        private void btnVMSaveState_Click(object sender, EventArgs e)
        {
            WriteOutput(1, "Saving state of Virtual Machine '" + objVirtualMachine.Name.ToString() + "'...");
            objVirtualMachine.Save();
        }

        private void btnRefreshVMGeneral_Click(object sender, EventArgs e)
        {
            RefreshTabInfo();
        }

        public void WriteOutput(int IndentationLevel, string MessageText)
        {
            // Simple method for writing output to the main form
            
            // Generate padding for output
            string BlankSpace = "";

            for (int i = 1; i < IndentationLevel; i++)
            {
                BlankSpace += "          ";
            }

            // Add output to main textbox
            txtOutput.Text += "\r\n" + BlankSpace + MessageText;
        }

        private void btnVMRemove_Click(object sender, EventArgs e)
        {
            objVirtualServer.UnregisterVirtualMachine(objVirtualMachine);
            WriteOutput(1, "Virtual Machine '" + objVirtualMachine.Name + " has been removed.");
            PopulateVMList();
        }
                
    }
}
